/*  CSP.h							(Lin  Zhong)
**								(1995 - '96)
**  Schnittstelle zu dem C-Modul, das Constraints ueber
**  einer partiell geordneten Struktur prueft
**
**
**
** 
**  Daten:
**  ------
**   Die partiell geordnete Struktur wird definiert durch einen
**   Menge von Elementen und binaeren Beziehungen zwischen diesen
**   Elementen ( a =< b)
**
**   Constraints enthalten konstante Elemente oder Variablen.
**   Es gibt binaere Constraints der Art x =< y und upper-bound-constraints
**   der Art  x = u \lub v \lub y \lub ...
**
**   Ablaufmodell:
**   Es wird zunaechst mit entsprechend vielen Aufrufen der Funktion 
**   order_def_less() die geordnete Struktur definiert.
**   Danach werden fuer ein bestimmtes ConstraintSatisfactionProblem (CSP)
**   die constraints definiert. Nachdem alle Constraints angegeben sind,
**   kann man ueber die Funktion solvable() pruefen, ob das CSP loesbar ist.
**   Nach dem Aufruf von reset_constraints() kann man zu der bereits 
**   definierten geordneten Struktur eine neue Menge von Constraints
**   definieren und auf Loesbarkeit pruefen.
*/

#include <stdio.h>

/* current restriction:
  ElemId and CVarId must be the same type
  and they should be some integer type
*/

typedef long unsigned ElemId;

/* 
**  Die Elemente der partiell geordneten Struktur werden 
**   durch Identifier vom Typ 'ElemId' repraesentiert 
*/

typedef long unsigned CVarId;


typedef void * UbExpr;   /* UpperBound Expression */

extern ElemId NULL_Elem;

extern CVarId *variables;
extern int count_variable;	
extern int count_solution;	
extern int *solutions;
extern ElemId *domain;	 
/*
** count_variable: a count for the variables;
** count_solution: a count for the solvings;
** variables[]: an array for the variables;
**  solutions[]: an array for the solvings.
*/

extern ElemId *neighbor,*newvalue;
/*
** neighbor[]: an array for the neighbors with less_neighbor();
** newvalue[]: an array for the new added constants produced by check_unique().
*/


extern void constant_num(int number);  
 /*the number of constants*/


extern void constant(ElemId e);


/*
**  Routinen zur Definition der geordneten Struktur 
*/

extern void order_num(int number);  
 /*the number of order-relations*/


extern void order_def_less(ElemId a, ElemId b);
/* 
**  es gelte a =< b 
*/


/*   Routinen zur Definition der Constraints */

extern void variable_num(int number);  
 /*the number of variables*/



extern void variable(CVarId x);



/* 
**  binaere constraints 
*/

extern void binary_constraint_num(int number);  
 /*the number of binary_constraints*/




extern void constraint_def_ee(ElemId a, ElemId b);  /* a =< b */
extern void constraint_def_ev(ElemId a, CVarId v);  /* a =< v */
extern void constraint_def_ve(CVarId u, ElemId b);  /* u =< b */
extern void constraint_def_vv(CVarId u, CVarId v);  /* u =< v */


/* 
**  upper-bound-constraint der Art
**       v = a \lub b \lub u \lub v ...
**   muessen aus den einzelnen Elementen zusammengebaut werden 
**   (\up steht fuer den upper-bound Operator) 
*/


extern void upper_bound_constraint_num(int number);  
 /*the number of  upper-bound-constraints*/


extern UbExpr ub_first_e(ElemId a);
extern UbExpr ub_first_v(CVarId a);
extern UbExpr ub_expr_e(UbExpr u, ElemId a); /* return: u \lub a */
extern UbExpr ub_expr_v(UbExpr u, CVarId v); /* return: u \lub v */


extern void constraint_def_eu(ElemId a, UbExpr u);  /* a = u */
extern void constraint_def_vu(CVarId v, UbExpr u);  /* v = u */


extern int check_unique();
/*
** int check_unique(): check whether those least-upper-bounds in 
** the given order-relations are unique, if no, print out some
** information and change the given order-relations with some new 
** constants into equivelent order-relations.
** return:0,unique; otherwise return: n, the number of new added constants.
*/


extern int solvable_first(void);
/*		return:  1    satisfiable  with first solution, 
** 			      then you can call  sovlable_all() to get all 
** 			      solutions.
**          	return:  0    not satisfiable.
**		return:  -1   error.	
**
*/



extern int solvable_all(void);
/*		return:  1    satisfiable  with all solutions
**          	return:  0    not satisfiable.
**		return:  -1   error.
*/


extern ElemId glb(ElemId a, ElemId b);
/*
** calculate the greatest-low-bound of element a and b,
** return: ElemId c, if c =a \glb b	
** return: NULL_Elem, if no c exist.
*/

extern ElemId lub(ElemId a, ElemId b);
/*
** calculate the least upper bound of element a and b,
** return: ElemId c, if c =a \lub b	
** return: NULL_Elem, if no c exist.
*/


extern int test_leq(ElemId x, ElemId y);
/*
** check whether there exist a order-relation x =< y in the given 
** order-relations; return: 1, if yes; return: 0, if no; return:-1, error.
*/

extern int is_maximal( ElemId a);
/*
** return: 1, if a is the maximal element in the given order relation, 
** 	      i.e. if a<=x, then a=x.
** return: 0, if  not.
*/

extern int is_minimal(ElemId a);
/*
** return: 1, if a is the minimal element in the given order relation,
**		i.e. if x>=a , then x=a.
** return: 0, if not.
*/

extern int less_neighbor(ElemId x);
/*
** To search the direct neighbor:y of x, with y<=x, x!=y, y is maximal.
** return: n, the number of neighbors of x;
** return: 0, no neighbor.
*/


extern void reset_constraints(void);
/* 
**  alle Constraints loeschen, damit man danach neue
**  Constraints definieren kann
*/

extern void free_space(void);
/* To release all the memory allocated*/


/*
** There is a special error element called 'NULL_Elem' additional to the normal ** elements in the domain of a constraint problem. This element is defined once ** using the function ' set_null_elem()' before the domain is defined. This 
** elenment is returned, e.g., by lub() and glb() to indicate an error or an 
** exception. 
*/

extern void set_null_elem(ElemId x);
/* The element 'x' is defined as the NULL_Elem */



extern long get_count_variable(void);
extern long get_count_solution(void);
extern CVarId  get_variables(long index);
extern ElemId get_solutions(long index1, long index2);
extern ElemId get_newvalue(long index);
extern ElemId get_neighbor(long index);

